unit LtrHead;
(* THIS UNIT PROVIDES FOR THE QUICK IMPLEMENTATION OF LETTER HEADS PRINTED
   ON THE FLY.  THE TLTRHEAD COMPONENT IS POINTED AT A TEXT FILE WITH THE
   DEFINITION OF THE BITMAPS, TEXT AND WINDOW AREAS AND THEN GIVEN A PRINTER

   -------------------------------------------------------
   COPYRIGHT     PETER FOX 2001
   AUTHOR        peter@eminent.demon.co.uk
   NOTE          Free but no warranties
   SOURCE        www.eminent.demon.co.uk/delphi.htm
   -------------------------------------------------------
   Documentation is provided in more detail in LtrHead.txt
   -------------------------------------------------------

*)

interface

uses
  useful,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, printers;

type
  TLhAlign = (lhaLeft,lhaRight,lhaCentre);
  TLhElementType = (lhetBitmap,lhetText,lhetWindow);


  { TLhElement is a just data record for bitmap/text or window
    all the printing intelligence is in TLtrHead }
  TLhElement = class(Tobject)
  private
    ftext       : string;   // possibly name of bitmap file
    { these fields will be accessed by to TLtrHead }
    falignment   : TLhAlign;
    felementtype : TlhElementType;
    ffont        : Tfont;
    fheight      : integer;     // dims in mm
    fleft        : integer;
    fname        : string;
    ftop         : integer;
    fwidth       : integer;
    fvalid       : boolean;     // set by create
    constructor create(const sourceLine:string);
    //function  getText:string;
    function  seteltext(const text:string):boolean;
  end;







  TLtrHead = class(TComponent)
  private
    fDefFile     : string;
    fdefPath     : string;
    fPrinterName : string;
    fReady       : boolean;
    fElements    : Tstringlist;
    fElementIx   : integer;
    f100mmPix    : integer;
    { whole object property handling }
    procedure SetDefinitionFile(const value:string);
    procedure SetDefinitionPath(const value:string);
    procedure SetPrinterName(const value:string);
    function  GetSelected:string;
    procedure SetSelected(const value:string);
    procedure TryToLoad;
    { selected element property handling }
    procedure setAlignment(const value:TLhAlign);
    function  getAlignment:TLhAlign;
    function  getBottom:Integer;
    function  getFont:Tfont;
    procedure setfont(value :tfont);
    function  getfontname:string;
    procedure setfontname(const value:string);
    function  getFontsize:Integer;
    procedure setFontSize(const value:integer);
    function  getHeight:Integer;
    function  getLeft:Integer;
    function  getRight:Integer;
    function  getSABPix:integer;
    function  getSABMm:integer;
    function  getText:string;
    procedure setText(const value:string);
    function  getTop:Integer;
    function  getWidth:Integer;
    { selected element private procedures }
    procedure elPrintBitmap;
    procedure elPrintText;
    procedure elPrintWindow;
  public
    constructor Create(AOwner: TComponent);override;
    destructor  Destroy;     override;
    { whole object methods }
    //function    CalibrationPrint:string;
    function    pix(const mm:integer):integer;
    function    pixTomm(const pixels:integer):integer;
    procedure   PrintElement(const elementName:string);
    procedure   PrintLetterhead;
    function    SelectPrinterNow:boolean;
    function    StartPrinting:boolean;
    { selected element methods }
    procedure   Print;
    procedure   PrintText(const someText:string);
  published
    { whole object properties }
    property DefinitionFile : string  read fDeffile        write setDefinitionFile;
    property Definitionpath  : string read fDefPath        write setDefinitionPath;
    property printername : string     read fprinterName    write setPrinterName ;
    property Ready : boolean          read fready;
    property Selected : string        read getSelected     write setSelected;
    { selected element properties }
    property Alignment : TLhAlign     read getAlignment    write setalignment;
    property Bottom    : integer      read getBottom;
    property Font      : Tfont        read getFont         write setfont;
    property FontName  : string       read getFontName     write setfontname;
    property FontSize  : integer      read getFontSize     write SetFontsize;
    property Height    : integer      read getHeight;
    property Left      : integer      read getLeft;
    property Right     : integer      read getRight;
    property SpaceAtBottomPix : integer read GetSABPix;
    property SpaceAtBottomMm : integer read GetSABMm;
    property Top       : integer      read getTop;
    property Text      : string       read getText         write SetText;
    property Width     : integer      read getWidth;
  end;

procedure Register;

implementation


constructor TLhElement.create(const sourceLine:string);
{ parse the source line
  Trailing comments will have been stripped already
  <name>=<element-type>:<parameters>
}
var
  e,s,w,f,a : string;
begin
  inherited create;
  fvalid := true;
  ffont := tfont.create;
  ffont.name := 'Arial';
  ffont.size := 10;
  falignment := lhaLeft;
  ftop := 10;
  fleft := 10;
  fheight := 30;
  fwidth := 30;

  // parse line
  s := sourceline;
  // name
  fname := uppercase(trim(chop(s,'=')));

  // element type
  e := uppercase(trim(chop(s,':')));
  case pos(e[1],'BTW') of
  0 : fvalid := false;
  1 : felementType := lhetBitmap;
  2 : felementType := lhetText;
  3 : felementType := lhetWindow;
  end;{case}
  if not fvalid then exit;

  // parameters
  fleft := strtointdef(trim(chop(s,',')),20);
  ftop  := strtointdef(trim(chop(s,',')),20);
  case felementType of
  lhetBitmap  : seteltext(trim(s));   // filename
  lhetText    : begin
                  // alignment
                  w := uppercase(trim(chop(s,',')));
                  if w <> '' then begin
                    a := chop(w,'=');
                    case pos(a,'LRC') of
                    0,1 : falignment := lhaLeft;
                    2   : falignment := lhaRight;
                    3   : falignment := lhaCentre;
                    end;{case}
                    fwidth := strtointdef(trim(chop(s,',')),40);
                  end;
                  // font name
                  f := trim(chop(s,','));
                  if f <> '' then ffont.name := f;
                  // font size
                  f := trim(chop(s,','));
                  if f <> '' then ffont.size := strtointdef(f,10);
                  // text
                  seteltext(trim(s));
                end;
  lhetWindow  : begin
                  fwidth  := strtointdef(trim(chop(s,',')),40);
                  fheight := strtointdef(trim(chop(s,',')),40);
                  s := trim(s);
                  if s <> '' then begin
                    // font name
                    f := trim(chop(s,','));
                    if f <> '' then ffont.name := f;
                  end;
                  if s <> '' then begin
                    // font size
                    f := trim(chop(s,','));
                    if f <> '' then ffont.size := strtointdef(f,10);
                  end;
                end;  
  end;{case}
end;







(*
function    TLhElement.getText:string;
{ what is the text (filename) }
begin
  result := ftext;
end;
*)

function    TLhElement.seteltext(const text:string):boolean;
{ setting the text
  do sanity check on bitmap filename }
begin
  ftext := text;
  result := true;
  if felementtype = lhetBitmap then begin
    if length(fText)<3 then result := false;
    if pos('.',ftext)=0 then result := false;
    if not result then fvalid := result;
  end;
end;








procedure Register;
begin
  RegisterComponents('Samples', [TLtrHead]);
end;







constructor TLtrHead.Create(AOwner: TComponent);
begin
  inherited create(aOwner);
  felements := Tstringlist.create;
  fdefPath := '';
  fDefFile := '';
  fPrintername := '';
  fready := false;
  fElementix := -1;
  f100mmPix := 0;
end;

destructor  TLtrHead.Destroy;
begin
  felements.free;
end;


procedure TLtrHead.SetDefinitionFile(const value:string);
{ set the definition file and see if we can load it }
begin
  if value = fDefFile then exit;  // no change
  fDefFile := value;
  TryToLoad;
end;

procedure TLtrHead.SetDefinitionPath(const value:string);
{ set the definition path and see if we can load the file }
var
  p : string;
begin
  // ensure path ends with a slash
  p := value;
  if p <> '' then begin
    if p[length(p)] <> '\' then p := p + '\';
  end;
  if p = fDefPath then exit;  // no change
  fDefPath := p;
  TryToLoad;
end;

procedure TLtrHead.SetPrinterName(const value:string);
{ set the printer name
  an empty name implies the default printer }
begin
  fprintername := value;
  //selectprinternow;  if we did this we would be warned about missing printer
end;


function  TLtrHead.GetSelected:string;
{ what is the name oi the selected element }
begin
  result := 'NO ELEMENT SELECTED';
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := name;
  end;
end;



procedure TLtrHead.SetSelected(const value:string);
{ locate the element by name }
begin
  fElementix := felements.indexof(uppercase(trim(value)));
end;

procedure TLtrHead.TryToLoad;
{ (1) can we find the definition file
  (2) if so try to read it

  Ready will be true if we can do both
  }
var
  i : integer;
  fn,s,ss,n : string;
  sl : Tstringlist;
begin
  // (1) can we find the definition file
  fReady := false;
  if fdeffile='' then exit;
  fn := fdefFile;
  if fdefpath <> '' then begin
    fn := fdefPath+fn;   //(fdefpath will have a training slash)
  end;
  if not fileexists(fn) then exit;

  // (2) lets try to read it
  for i := 0 to felements.count-1 do TLhElement(felements.objects[i]).free;
  felements.clear;
  sl := tstringlist.create;
  TRY
    sl.loadfromfile(fn);
    for i := 0 to sl.count-1 do begin
      s := trim(sl.strings[i]);
      s := substr(s,';;','<SEMI-COLON>'); // allow ';' to appear in text if specified as ";;"
      ss := trim(chop(s,';'));
      if ss = '' then continue;
      if pos('=',ss)=0 then continue;
      if pos(':',ss)=0 then continue;
      if pos(',',ss)=0 then continue;
      s := substr(ss,'<SEMI-COLON>',';');
      // s looks like what we want
      ss := s;
      n := trim(uppercase(chop(ss,'=')));
      felements.addobject(n,TLhElement.create(s));
    end;
  FINALLY
    sl.free;
  END;

  // flag the result
  fready := (felements.count >0);

end;


procedure TLtrHead.setAlignment(const value:TLhAlign);
{ try to set the alignment }
begin
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    falignment := value;  // doesn't apply to bitmaps but who cares
  end;
end;

function  TLtrHead.getAlignment:TLhAlign;
{ returieve alignment }
begin
  result := lhaLeft;   // default
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := falignment;
  end;
end;

function  TLtrHead.getBottom:Integer;
{ bottom in mm }
begin
  result := pix(50);
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := pix(ftop+fheight);
  end;
end;

function  TLtrHead.getFont:Tfont;
{ return the element font }
begin
  result := nil;
  if felementix = -1 then begin  // nothing selected
    if ready then begin
      result.assign(TLhElement(felements.objects[0]).ffont);
      exit;
    end;
  end;
  // normal case
  with felements.objects[felementix] as TLhElement do begin
    // not applicable to bitmaps but who cares - we have a default
    result.assign(ffont);
  end;
end;

procedure TLtrHead.setfont(value : tfont);
{ set element font }
begin
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    // not applicable to bitmaps but who cares
    ffont.assign(value);
  end;
end;

function  TLtrHead.getfontname:string;
{ get element font name }
begin
  result := 'Arial';
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
     result := ffont.name;
  end;
end;

procedure TLtrHead.setfontname(const value:string);
{ set element font name }
begin
  if felementix = -1 then exit;  // nothing selected
  if value = '' then exit;       // silly
  with felements.objects[felementix] as TLhElement do begin
    ffont.name := value;
  end;
end;

function  TLtrHead.getFontsize:Integer;
{ get element font size }
begin
  result := 10;
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := ffont.size;
  end;
end;

procedure TLtrHead.setFontSize(const value:integer);
{ set element font size }
begin
  if felementix = -1 then exit;  // nothing selected
  if value < 4 then exit;        // silly
  with felements.objects[felementix] as TLhElement do begin
    ffont.size := value;
  end;
end;

function  TLtrHead.getHeight:Integer;
{ get element height in pixels }
begin
  result := pix(20);
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := pix(fheight);
  end;
end;

function  TLtrHead.getLeft:Integer;
{ get element left in pix }
begin
  result := pix(20);
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := pix(fLeft);
  end;
end;

function  TLtrHead.getRight:Integer;
{ get right extent of element }
begin
  result := pix(100);
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := pix(fleft+fwidth);
  end;
end;

function  TLtrHead.getSABPix:integer;
{ get space remaining from current printer position to bottom
  of current element.  Result in pixels}
var
  pp,eb : integer;
begin
  result := 0;
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
  // only really applies to window elements but who cares
    pp := printer.canvas.penpos.y;
    eb := bottom;
    result := eb-pp;
  end;
end;

function  TLtrHead.getSABMm:integer;
{ get space remaining from current printer position to bottom
  of current element.  Result in milimetres}
var
  p : integer;
begin
  p := getSABPix;    // get the distance in pixels
  result := pixtomm(p);
end;

function  TLtrHead.getText:string;
{ return the text of the element
  In the case of a bitmap the text is the filename }
begin
  result := 'NO ELEMENT';
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := ftext;
  end;
end;

procedure TLtrHead.setText(const value:string);
{ set element text
  in the case of a bitmap the text is the filename }
begin
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    seteltext(value);
  end;
end;

function  TLtrHead.getTop:Integer;
{ get top of element in pix }
begin
  result := pix(15);
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := pix(ftop);
  end;
end;

function  TLtrHead.getWidth:Integer;
{ get width of element in pix }
begin
  result := pix(44);
  if felementix = -1 then exit;  // nothing selected
  with felements.objects[felementix] as TLhElement do begin
    result := pix(fwidth);
  end;
end;


function    TLtrHead.pix(const mm:integer):integer;
{ convert mm to pix}
begin
  result := (mm * f100mmPix) div 100;
end;

function    TLtrHead.pixTomm(const pixels:integer):integer;
{ inverse of pix() }
begin
  result := (pixels * 100 ) div f100mmPix;
end;

procedure   TLtrHead.PrintElement(const elementName:string);
{ print the named element }
begin
  SetSelected(elementName);
  print;
end;

procedure   TLtrHead.PrintLetterhead;
{ initialise printing and
  print all the items in the letterhead }
var
  i : integer;
begin
  if not startprinting then exit;
  for i := 0 to felements.count-1 do begin
    felementix := i;
    print;
  end;
end;




function    TLtrHead.SelectPrinterNow:boolean;
{ try to select printer
  use default if no printer name specified
  get the pix() for the new printer }
begin
  result := true;
  // trap missing printer
  if fprinterName = '' then begin
    // use default printer
    printer.printerindex := -1;
  end else begin
    result := selectnamedprinter(fprintername);
  end;


  if not result then begin
    showmessage('Cannot find a printer called "'+fprintername+'"');
  end else begin
    f100mmpix := useful.pix(100);
  end;
end;


function    TLtrHead.StartPrinting:boolean;
{ see if we can initialise the printer
  return false if a serious problem }
begin
  result := false;

  // safety check
  if not ready then begin
    showmessage('ERROR:Letterhead is not ready');
    exit;
  end;

  // close any inadvertently open print job
  if printer.printing then begin
    try
      printer.enddoc;
      result := true;
    except
      showmessage('ERROR:Cannot finish previous print job');
    end;
    if result = false then exit;
  end;

  // select printer and get pix etc
  result := selectPrinterNow;
  if result = false then exit;

  // start document
  try
    printer.begindoc
  except
    showmessage('ERROR|Printer won''t start');  // !!!
    result := false;
  end;
end;


procedure   TLtrHead.Print;
{ print the selected element }
begin
  if felementix = -1 then exit;  // nothing selected

  if not printer.printing then printer.begindoc;  // safety

  with felements.objects[felementix] as TLhElement do begin
    if ftext = '' then exit;
    if not fvalid then exit;

    case felementtype of
    lhetBitmap:  elprintbitmap;
    lhetText:    elprinttext;
    lhetWindow:  elprintwindow;
    end{case}

  end;{with tlhelement}
end;

procedure   TLtrHead.PrintText(const someText:string);
{ set text then print
  if a bitmap then the text is the filename }
begin
  if felementix < 0 then exit;
  settext(sometext);
  print;
end;


procedure TLtrHead.elPrintBitmap;
{ print element bitmap }
var
  bm : tbitmap;
  f,fn,ffn,ffnp,errmsg,t : string;
begin
  // locate the bitmap file
  bm := nil;
  errmsg := '';
  f := '';
  fn := text;                                 // filename
  ffn := fDefPath+fn;                         // full file name
  ffnp := substr(ffn,'.',fPrinterName+'.');   // possibly with printer name extension
  if fileexists(ffnp) then begin
    f := ffnp;
  end else begin
    if fileexists(ffn) then f := ffn;
  end;

  if f = '' then begin
    errmsg := 'ERROR '+ffn+' MISSING';
  end else begin
    // load the file
    bm := tbitmap.create;
    try
      bm.loadfromfile(f);
    except
      errmsg := 'ERROR CANNOT LOAD BITMAP|'+f;
    end;
  end;

  if errmsg = '' then begin
    if assigned(bm) then printer.canvas.draw(left,top,bm);
  end else begin
    t := text;
    text := errmsg;
    elPrintText;
    text := t;
  end;
end;

procedure TLtrHead.elPrintText;
{ print element text }
var
  x,v : integer;
  s,t : string;
begin
  t := text;
  v := top;
  //f := font;
  //printer.canvas.font.assign(f);
  printer.canvas.font.name := fontname;
  printer.canvas.font.size := fontsize;

  with printer.canvas do begin
    while t <> '' do begin
      s := chop(t,'|');
      case alignment of
      lhaLeft   : begin
                    textout(left,v,s);
                  end;
      lhaRight  : begin
                    x := textwidth(s);
                    textout(right-x,v,s);
                  end;
      lhaCentre : begin
                    x := textwidth(s);
                    textout(left+((width-x) div 2),v,s);
                  end;
      end;
      v := v + textheight(s);
    end;
  end;
end;


procedure TLtrHead.elPrintWindow;
{ print window text }
begin
  elprinttext;  // do the same as text
end;




end.

